/**
 * Copyright 2002, 2014 IBM Corp.
 * 
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 * 
 *  Author: Sami Salkosuo, sami.salkosuo@fi.ibm.com
 */
package clenotes.commands;

import clenotes.Command;
import clenotes.CommandLineArguments;
import clenotes.Logger;
import clenotes.commands.ReadMail;
import clenotes.utils.DominoUtils;
import com.google.common.base.Objects;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.util.List;
import lotus.domino.Database;
import lotus.domino.DateTime;
import lotus.domino.Document;
import lotus.domino.DocumentCollection;
import lotus.domino.Session;
import org.eclipse.xtext.xbase.lib.Exceptions;
import org.eclipse.xtext.xbase.lib.InputOutput;

@SuppressWarnings("all")
public class Search {
  public static int execute(final Command cmd, final Session notesSession) {
    try {
      Database mailDb = DominoUtils.getMailDatabase(notesSession);
      DateTime dt = notesSession.createDateTime("Yesterday");
      String adjustDay = CommandLineArguments.getValue("adjust-day");
      boolean _notEquals = (!Objects.equal(adjustDay, null));
      if (_notEquals) {
        int _parseInt = Integer.parseInt(adjustDay);
        int _minus = (-_parseInt);
        dt.adjustDay(_minus);
      }
      String subject = cmd.getOptionValue("subject");
      String sender = cmd.getOptionValue("from");
      String formula = cmd.getOptionValue("formula");
      String formulaFile = cmd.getOptionValue("formula-file");
      String fulltext = cmd.getOptionValue("fulltext");
      boolean _and = false;
      boolean _and_1 = false;
      boolean _and_2 = false;
      boolean _and_3 = false;
      boolean _equals = Objects.equal(subject, null);
      if (!_equals) {
        _and_3 = false;
      } else {
        boolean _equals_1 = Objects.equal(sender, null);
        _and_3 = _equals_1;
      }
      if (!_and_3) {
        _and_2 = false;
      } else {
        boolean _equals_2 = Objects.equal(formula, null);
        _and_2 = _equals_2;
      }
      if (!_and_2) {
        _and_1 = false;
      } else {
        boolean _equals_3 = Objects.equal(formulaFile, null);
        _and_1 = _equals_3;
      }
      if (!_and_1) {
        _and = false;
      } else {
        boolean _equals_4 = Objects.equal(fulltext, null);
        _and = _equals_4;
      }
      if (_and) {
        InputOutput.<String>println("[ERROR] Search option must be specified.");
        return (-101);
      }
      String mailSearchString = null;
      boolean _notEquals_1 = (!Objects.equal(subject, null));
      if (_notEquals_1) {
        String _commonUserName = notesSession.getCommonUserName();
        String _format = String.format("@Contains(Subject;\"%s\") & !@Contains(From;\"%s\")", subject, _commonUserName);
        mailSearchString = _format;
      }
      boolean _notEquals_2 = (!Objects.equal(sender, null));
      if (_notEquals_2) {
        String _format_1 = String.format("@Contains(From;\"%s\") | @Contains(Principal;\"%s\")", sender, sender);
        mailSearchString = _format_1;
      }
      boolean _notEquals_3 = (!Objects.equal(formulaFile, null));
      if (_notEquals_3) {
        File file = new File(formulaFile);
        boolean _exists = file.exists();
        if (_exists) {
          FileInputStream input = new FileInputStream(file);
          ByteArrayOutputStream output = new ByteArrayOutputStream();
          int chr = input.read();
          boolean _while = (chr > (-1));
          while (_while) {
            {
              output.write(chr);
              int _read = input.read();
              chr = _read;
            }
            _while = (chr > (-1));
          }
          input.close();
          output.close();
          String _string = output.toString();
          formula = _string;
        } else {
          InputOutput.<String>println((("[ERROR] File does not exist: " + file) + "."));
        }
      }
      boolean _notEquals_4 = (!Objects.equal(formula, null));
      if (_notEquals_4) {
        Logger.log(("Formula: " + formula));
        mailSearchString = formula;
      }
      DocumentCollection docCollection = null;
      boolean _notEquals_5 = (!Objects.equal(fulltext, null));
      if (_notEquals_5) {
        Logger.log(("Full text search: " + fulltext));
        DocumentCollection _FTSearch = mailDb.FTSearch(fulltext);
        docCollection = _FTSearch;
      } else {
        Logger.log(("Mail search string: " + mailSearchString));
        DocumentCollection _search = mailDb.search(mailSearchString);
        docCollection = _search;
      }
      DocumentCollection _sortDocumentCollectionIfNecessary = DominoUtils.sortDocumentCollectionIfNecessary(docCollection, mailDb, cmd);
      docCollection = _sortDocumentCollectionIfNecessary;
      int matches = docCollection.getCount();
      Logger.log(("Number of documents found: " + Integer.valueOf(matches)));
      String readIndex = cmd.getOptionValue("read");
      boolean _equals_5 = Objects.equal(readIndex, "*");
      if (_equals_5) {
        int n = 1;
        Document doc = docCollection.getNthDocument(n);
        boolean _notEquals_6 = (!Objects.equal(doc, null));
        boolean _while_1 = _notEquals_6;
        while (_while_1) {
          {
            boolean isDeleted = ReadMail.readMail(doc, cmd, notesSession);
            n = (n + 1);
            doc.recycle();
            Document _nthDocument = docCollection.getNthDocument(n);
            doc = _nthDocument;
          }
          boolean _notEquals_7 = (!Objects.equal(doc, null));
          _while_1 = _notEquals_7;
        }
      } else {
        int _readIndex = (-1);
        boolean _notEquals_7 = (!Objects.equal(readIndex, null));
        if (_notEquals_7) {
          int _parseInt_1 = Integer.parseInt(readIndex);
          _readIndex = _parseInt_1;
        }
        List<? extends Object> values = DominoUtils.getMails(docCollection, cmd, _readIndex, false, notesSession);
        Object numberOfMails = values.get(0);
        Object _get = values.get(1);
        String outputTxt = ((String) _get);
        Object _get_1 = values.get(2);
        Document mailDoc = ((Document) _get_1);
        boolean _equals_6 = Objects.equal(mailDoc, null);
        if (_equals_6) {
          String txt = "mails";
          boolean _equals_7 = Objects.equal(numberOfMails, Integer.valueOf(1));
          if (_equals_7) {
            txt = "mail";
          }
          InputOutput.<String>println((((("Search returned " + numberOfMails) + " ") + txt) + ".\r\n"));
          InputOutput.<String>println(outputTxt);
        } else {
          ReadMail.readMail(mailDoc, cmd, notesSession);
        }
        boolean _notEquals_8 = (!Objects.equal(mailDoc, null));
        if (_notEquals_8) {
          mailDoc.recycle();
        }
      }
      return 0;
    } catch (Throwable _e) {
      throw Exceptions.sneakyThrow(_e);
    }
  }
}
