/*
 * Copyright 2002, 2014 IBM Corp.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.  
 *
 *  Author: Sami Salkosuo, sami.salkosuo@fi.ibm.com
*/
package clenotes

import clenotes.commands.Appointments
import clenotes.commands.List
import clenotes.commands.MailDBInfo
import clenotes.commands.ReadMail
import clenotes.commands.Replicate
import clenotes.commands.Search
import clenotes.commands.Send
import clenotes.commands.Today
import lotus.domino.NotesFactory
import lotus.domino.Session

class CLENotes implements Runnable {

	var returnCode = 0
	var Session notesSession = null

	new() {
	}

	override run() {

		if (!verifyOptionsAndCommands()) {
			returnCode = -1
			return
		}

		var commandMap = CommandLineArguments::commandMap

		if (CommandLineArguments::hasOption("version")) {
			//show version info of CLENotes and Java
			option_Version
			return
		}

		//start notes session
		var password = CommandLineArguments::getValue("password")

		//TODO: add DIIOP
		//--server-host!: Domino server host name or IP address to be used with DIIOP calls (does not require local Notes installation).
		var String serverHost = null // CommandLineArguments::getValue("server-host")

		if (password != null) {
			notesSession = NotesFactory::createSession(serverHost, null as String, password)

		} else {
			notesSession = NotesFactory::createSession()

		//notesSession = NotesFactory::createSessionWithFullAccess()
		}

		//do commands	
		if (!commandMap.empty) {

			for (cmdName : commandMap.keySet) {

				//do commands one at a time
				var command = commandMap.get(cmdName) as Command
				switch (cmdName) {
					//use simple switch to determine command object to invoke
					//would be better to dynamically find out the command object based
					//on command name... but perhaps later...					
					case "today": {
						Today::execute(command, notesSession)
					}
					case "replicate": {
						Replicate::execute(command, notesSession)
					}
					case "search": {
						Search::execute(command, notesSession)

					}
					case "read": {
						ReadMail::readLatestMail(command, notesSession)
					}
					case "list": {
						List::execute(command, notesSession)

					}
					case "notes-version": {
						command_NotesVersion

					}
					case "send": {
						Send::execute(command, notesSession)

					}
					case "appointments": {
						Appointments::execute(command, notesSession)

					}
					case "maildbinfo": {
						MailDBInfo::execute(command, notesSession)
					}
				}

			}
		}
		notesSession.recycle;
	}

	def option_Version() {
		println("CLENotes      : " + Configuration::VERSION)
		println("Java          : " + System::getProperty("java.version"))

	}

	def command_NotesVersion() {
		var String prog
		if (notesSession.onServer) {
			prog = 'Domino version: '

		} else {
			prog = 'Notes version : '

		}
		print(prog)
		print(notesSession.getNotesVersion())
		print(" (")
		print(notesSession.getPlatform)
		println(")")
		println("CLENotes      : " + Configuration::VERSION)
		println("Java          : " + System::getProperty("java.version"))

	}

	def verifyOptionsAndCommands() {

		//verify that options and commands specified in command line
		//are valid
		var atLeastOneValid = false
		var optionMap = CommandLineArguments::globalOptionMap
		var optionMapConfig = Configuration::globalOptionMap
		for (key : optionMap.keySet) {
			if (optionMapConfig.containsKey(key)) {
				atLeastOneValid = true
			} else {
				println("Unknown option: " + key)
			}
		}

		var commandMap = CommandLineArguments::commandMap
		var commandMapConfig = Configuration::commandMap
		for (key : commandMap.keySet) {
			if (commandMapConfig.containsKey(key)) {
				atLeastOneValid = true
			} else {
				println("Unknown command: " + key)
			}
		}

		atLeastOneValid
	}

	def getReturnCode() {
		returnCode
	}

}
